info = {
    'name': 'CrossingTraffic_POMDP',
    'description': 'In a grid, a robot must get to a goal and avoid obstacles arriving randomly and moving left. This is the pomdp version',
    'context': 'ippc2014',
    'tags': '',
    'viz': 'None'
}