info = {
    'name': 'Elevators',
    'description': 'The Elevator domain models evening rush hours when people from different floors in a building want to go down to the bottom floor using elevators.',
    'context': '',
    'tags': '',
    'viz': 'ElevatorViz.ElevatorVisualizer'
}