info = {
    'name': 'RedFinnedBlueEye',
    'description': 'The Red-finned Blue-eye domain tackles the problem of eradicating the invasive Gambusia from the habitat of the red-finned blue-eye.',
    'context': 'ippc2018',
    'tags': '',
    'viz': 'None'
}
