info = {
    'name': 'Manufacturer',
    'description': 'In this domain, the agent manages a manufacturing company that buys goods to use them in the production of other goods.',
    'context': 'ippc2018',
    'tags': '',
    'viz': 'None'
}
