

info = {
    'name': 'MarsRover',
    'description': 'Multi Rover Navigation, where a group of agent needs to harvest mineral.',
    'context': '',
    'tags': '',
    'viz': 'MarsRoverViz.MarsRoverVisualizer'
}