
info = {
    'name': 'SysAdmin_POMDP',
    'description': 'An example RDDL description for the well-known SysAdmin problem. This is the pomdp version',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}