info = {
    'name': 'MountainCar',
    'description': 'A simple continuous MDP for the classical mountain car control problem.',
    'context': '',
    'tags': '',
    'viz': 'MountainCarViz.MountainCarVisualizer'
}