info = {
    'name': 'GameOfLife_POMDP',
    'description': 'A simple DBN to encode Conway\'s cellular automata game of life on a grid. This is the pomdp version.',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}