class RDDLRepoEnvironmentNotExist(ValueError):
    pass

class RDDLRepoInstanceNotExist(ValueError):
    pass

class RDDLRepoDomainNotExist(ValueError):
    pass

class RDDLRepoProblemDuplication(ValueError):
    pass

class RDDLRepoUnresolvedDependency(ValueError):
    pass

class RDDLRepoManifestEmpty(ValueError):
    pass

class RDDLRepoContextNotExist(ValueError):
    pass
