info = {
    'name': 'AcademicAdvising_POMDP',
    'description': 'In this domain, a student may take courses at a given cost and passes the course with a probability determined by how many of the prerequisites they have successfully passed. This is the pomdp version',
    'context': 'ippc2014',
    'tags': '',
    'viz': 'None'
}