info = {
    'name': 'Elevators_POMDP',
    'description': 'This domain has a number of elevators delivering passengers to either the top or the bottom floor. This is the pomdp version.',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}