info = {
    'name': 'EarthObservation',
    'description': 'The Earth Observation domain models a satellite orbiting Earth that can take pictures of the landscape below with a camera.',
    'context': 'ippc2018',
    'tags': '',
    'viz': 'None'
}
