info = {
    'name': 'UAV_Discrete',
    'description': 'Discrete action space version of multi-UAV problem where a group of UAVs have to reach goal positions in the 3d Space.',
    'context': '',
    'tags': '',
    'viz': 'UAVsViz.UAVsVisualizer'
}