info = {
    'name': 'Traffic_POMDP',
    'description': 'A simple binary version of the cell transition model (CTM) for modeling traffic. This is the pomdp verion',
    'context': 'ippc2014',
    'tags': '',
    'viz': 'None'
}
