info = {
    'name': 'CooperativeRecon_MDP',
    'description': 'There is a 2d grid with an agent, a base, some hazard squares, and objects in different locations.',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}