info = {
    'name': 'ChromaticDice',
    'description': 'Chromatic Dice is a variant of the popular dice game Yahtzee (also known as Kniffel).',
    'context': 'ippc2018',
    'tags': '',
    'viz': 'None'
}
