info = {
    'name': 'Navigation_MDP',
    'description': 'In a grid, a robot must get to a goal G, and every cell offers the robot a (different) chance of disappearing.',
    'context': 'ippc2011',
    'tags': '',
    'viz': 'None'
}